-- Grab environment we need
local ipairs = ipairs
local math = math

--- The gridh layout layoutbox icon.
-- @beautiful beautiful.layout_gridh
-- @param surface
-- @see gears.surface

--- The gridv layout layoutbox icon.
-- @beautiful beautiful.layout_gridv
-- @param surface
-- @see gears.surface

local grid = {}

local function do_grid(p, orientation)
    local wa = p.workarea
    local cls = p.clients

    -- Swap workarea dimensions, if our orientation is "east"
    if orientation == 'east' then
        wa.width, wa.height = wa.height, wa.width
        wa.x, wa.y = wa.y, wa.x
    end

    if #cls > 0 then
        local rows, cols
        if #cls == 2 then
            rows, cols = 1, 2
        else
            rows = math.ceil(math.sqrt(#cls))
            cols = math.ceil(#cls / rows)
        end
        local first_col_rows = #cls - rows * (cols - 1)
        for k, c in ipairs(cls) do
            k = k - 1
            local g = {}

            local row, col, lrows, lcols
            if k < first_col_rows then
                row = k
                col = 0
                lrows = first_col_rows
                lcols = cols
            else
                row = (k - first_col_rows) % rows
                col = math.floor((k - first_col_rows) / rows) + 1
                lrows = rows
                lcols = cols
            end

            if row == lrows - 1 then
                g.height = wa.height - math.ceil(wa.height / lrows) * row
                g.y = wa.height - g.height
            else
                g.height = math.ceil(wa.height / lrows)
                g.y = g.height * row
            end

            if col == lcols - 1 then
                g.width = wa.width - math.ceil(wa.width / lcols) * col
                g.x = wa.width - g.width
            else
                g.width = math.ceil(wa.width / lcols)
                g.x = g.width * col
            end

            g.y = g.y + wa.y
            g.x = g.x + wa.x

            -- Swap window dimensions, if our orientation is "east"
            if orientation == 'east' then
                g.width, g.height = g.height, g.width
                g.x, g.y = g.y, g.x
            end

            p.geometries[c] = g
        end
    end
end

-- Horizontal grid layout.
-- @param screen The screen to arrange.
grid.horizontal = {}
grid.horizontal.name = "gridh"

function grid.horizontal.arrange(p)
    return do_grid(p, "east")
end

-- Vertical grid layout.
-- @param screen The screen to arrange.
grid.name = "grid"
function grid.arrange(p)
    return do_grid(p, "south")
end

--- The grid layout.
-- Try to give all clients the same size.
-- @clientlayout awful.layout.suit.grid
-- @usebeautiful beautiful.layout_gridv

--- The horizontal grid layout.
-- Try to give all clients the same size.
-- @clientlayout awful.layout.suit.grid.horizontal
-- @usebeautiful beautiful.layout_gridh

return grid
